@@ -92,6 +92,7 @@ import Data.Word (Word8)
 import GHC.IO (noDuplicate)
 
 import qualified Codec.Compression.Zlib.Stream as Stream
+import Codec.Compression.Zlib.ByteStringCompat (mkBS, withBS)
 import Codec.Compression.Zlib.Stream (Stream)
 
 -- | The full set of parameters for compression. The defaults are
@@ -100,7 +101,7 @@ import Codec.Compression.Zlib.Stream (Stream)
 -- The 'compressBufferSize' is the size of the first output buffer containing
 -- the compressed data. If you know an approximate upper bound on the size of
 -- the compressed data then setting this parameter can save memory. The default
--- compression output buffer size is @16k@. If your extimate is wrong it does
+-- compression output buffer size is @16k@. If your estimate is wrong it does
 -- not matter too much, the default buffer size will be used for the remaining
 -- chunks.
 --
@@ -121,12 +122,12 @@ data CompressParams = CompressParams {
 -- containing the uncompressed data. If you know an exact or approximate upper
 -- bound on the size of the decompressed data then setting this parameter can
 -- save memory. The default decompression output buffer size is @32k@. If your
--- extimate is wrong it does not matter too much, the default buffer size will
+-- estimate is wrong it does not matter too much, the default buffer size will
 -- be used for the remaining chunks.
 --
 -- One particular use case for setting the 'decompressBufferSize' is if you
 -- know the exact size of the decompressed data and want to produce a strict
--- 'Data.ByteString.ByteString'. The compression and deccompression functions
+-- 'Data.ByteString.ByteString'. The compression and decompression functions
 -- use lazy 'Data.ByteString.Lazy.ByteString's but if you set the
 -- 'decompressBufferSize' correctly then you can generate a lazy
 -- 'Data.ByteString.Lazy.ByteString' with exactly one chunk, which can be
@@ -229,7 +230,7 @@ data DecompressError =
      -- get this error, for example if the input data just isn't a compressed
      -- zlib data stream. In particular if the data checksum turns out to be
      -- wrong then you will get all the decompressed data but this error at the
-     -- end, instead of the normal sucessful 'StreamEnd'.
+     -- end, instead of the normal successful 'StreamEnd'.
    | DataFormatError String
   deriving (Eq, Typeable)
 
@@ -306,7 +307,7 @@ foldDecompressStreamWithInput chunk end err = \s lbs ->
 -- that they can produce output without demanding all input, however they need
 -- the input data stream as a lazy 'L.ByteString'. Having the input data
 -- stream as a lazy 'L.ByteString' often requires using lazy I\/O which is not
--- appropriate in all cicumstances.
+-- appropriate in all circumstances.
 --
 -- For these cases an incremental interface is more appropriate. This interface
 -- allows both incremental input and output. Chunks of input data are supplied
@@ -471,13 +472,12 @@ compressStream format (CompressParams compLevel method bits memLevel
     \chunk -> do
       Stream.deflateInit format compLevel method bits memLevel strategy
       setDictionary mdict
-      case chunk of
-        _ | S.null chunk ->
-          fillBuffers 20   --gzip header is 20 bytes, others even smaller
-
-        S.PS inFPtr offset length -> do
-          Stream.pushInputBuffer inFPtr offset length
-          fillBuffers initChunkSize
+      withBS chunk $ \inFPtr length ->
+        if length == 0
+          then fillBuffers 20   --gzip header is 20 bytes, others even smaller
+          else do
+            Stream.pushInputBuffer inFPtr 0 length
+            fillBuffers initChunkSize
 
   where
     -- we flick between two states:
@@ -492,8 +492,8 @@ compressStream format (CompressParams compLevel method bits memLevel
     Stream.consistencyCheck
 #endif
 
-    -- in this state there are two possabilities:
-    --   * no outbut buffer space is available
+    -- in this state there are two possibilities:
+    --   * no output buffer space is available
     --       - in which case we must make more available
     --   * no input buffer is available
     --       - in which case we must supply more
@@ -507,11 +507,11 @@ compressStream format (CompressParams compLevel method bits memLevel
       Stream.pushOutputBuffer outFPtr 0 outChunkSize
 
     if inputBufferEmpty
-      then return $ CompressInputRequired $ \chunk ->
-           case chunk of
-             _ | S.null chunk          -> drainBuffers True
-             S.PS inFPtr offset length -> do
-                Stream.pushInputBuffer inFPtr offset length
+      then return $ CompressInputRequired $ flip withBS $ \inFPtr length ->
+           if length == 0
+             then drainBuffers True
+             else do
+                Stream.pushInputBuffer inFPtr 0 length
                 drainBuffers False
       else drainBuffers False
 
@@ -534,7 +534,7 @@ compressStream format (CompressParams compLevel method bits memLevel
         outputBufferFull <- Stream.outputBufferFull
         if outputBufferFull
           then do (outFPtr, offset, length) <- Stream.popOutputBuffer
-                  let chunk = S.PS outFPtr offset length
+                  let chunk = mkBS outFPtr offset length
                   return $ CompressOutputAvailable chunk $ do
                     fillBuffers defaultCompressBufferSize
           else do fillBuffers defaultCompressBufferSize
@@ -545,7 +545,7 @@ compressStream format (CompressParams compLevel method bits memLevel
         outputBufferBytesAvailable <- Stream.outputBufferBytesAvailable
         if outputBufferBytesAvailable > 0
           then do (outFPtr, offset, length) <- Stream.popOutputBuffer
-                  let chunk = S.PS outFPtr offset length
+                  let chunk = mkBS outFPtr offset length
                   Stream.finalise
                   return $ CompressOutputAvailable chunk (return CompressStreamEnd)
           else do Stream.finalise
@@ -607,25 +607,25 @@ decompressStream format (DecompressParams bits initChunkSize mdict allMembers)
                        Stream.inflateReset
                   else assert outputBufferFull $
                        Stream.inflateInit format bits
-      case chunk of
-        _ | S.null chunk -> do
-          -- special case to avoid demanding more input again
-          -- always an error anyway
-          when outputBufferFull $ do
-            let outChunkSize = 1
-            outFPtr <- Stream.unsafeLiftIO (S.mallocByteString outChunkSize)
-            Stream.pushOutputBuffer outFPtr 0 outChunkSize
-          drainBuffers True
-
-        S.PS inFPtr offset length -> do
-          Stream.pushInputBuffer inFPtr offset length
-          -- Normally we start with no output buffer (so counts as full) but
-          -- if we're resuming then we'll usually still have output buffer
-          -- space available
-          assert (if not resume then outputBufferFull else True) $ return ()
-          if outputBufferFull
-            then fillBuffers initChunkSize
-            else drainBuffers False
+      withBS chunk $ \inFPtr length ->
+        if length == 0
+          then do
+            -- special case to avoid demanding more input again
+            -- always an error anyway
+            when outputBufferFull $ do
+              let outChunkSize = 1
+              outFPtr <- Stream.unsafeLiftIO (S.mallocByteString outChunkSize)
+              Stream.pushOutputBuffer outFPtr 0 outChunkSize
+            drainBuffers True
+          else do
+            Stream.pushInputBuffer inFPtr 0 length
+            -- Normally we start with no output buffer (so counts as full) but
+            -- if we're resuming then we'll usually still have output buffer
+            -- space available
+            assert (if not resume then outputBufferFull else True) $ return ()
+            if outputBufferFull
+              then fillBuffers initChunkSize
+              else drainBuffers False
 
   where
     -- we flick between two states:
@@ -641,8 +641,8 @@ decompressStream format (DecompressParams bits initChunkSize mdict allMembers)
     Stream.consistencyCheck
 #endif
 
-    -- in this state there are two possabilities:
-    --   * no outbut buffer space is available
+    -- in this state there are two possibilities:
+    --   * no output buffer space is available
     --       - in which case we must make more available
     --   * no input buffer is available
     --       - in which case we must supply more
@@ -657,11 +657,12 @@ decompressStream format (DecompressParams bits initChunkSize mdict allMembers)
 
     if inputBufferEmpty
       then return $ DecompressInputRequired $ \chunk ->
-           case chunk of
-             _ | S.null chunk          -> drainBuffers True
-             S.PS inFPtr offset length -> do
-                Stream.pushInputBuffer inFPtr offset length
-                drainBuffers False
+           withBS chunk $ \inFPtr length ->
+             if length == 0
+               then drainBuffers True
+               else do
+                 Stream.pushInputBuffer inFPtr 0 length
+                 drainBuffers False
       else drainBuffers False
 
 
@@ -682,7 +683,7 @@ decompressStream format (DecompressParams bits initChunkSize mdict allMembers)
         outputBufferFull <- Stream.outputBufferFull
         if outputBufferFull
           then do (outFPtr, offset, length) <- Stream.popOutputBuffer
-                  let chunk = S.PS outFPtr offset length
+                  let chunk = mkBS outFPtr offset length
                   return $ DecompressOutputAvailable chunk $ do
                     fillBuffers defaultDecompressBufferSize
           else do fillBuffers defaultDecompressBufferSize
@@ -690,12 +691,12 @@ decompressStream format (DecompressParams bits initChunkSize mdict allMembers)
       Stream.StreamEnd      -> do
         -- The decompressor tells us we're done.
         -- Note that there may be input bytes still available if the stream is
-        -- embeded in some other data stream, so we return any trailing data.
+        -- embedded in some other data stream, so we return any trailing data.
         inputBufferEmpty <- Stream.inputBufferEmpty
         if inputBufferEmpty
           then do finish (DecompressStreamEnd S.empty)
           else do (inFPtr, offset, length) <- Stream.popRemainingInputBuffer
-                  let inchunk = S.PS inFPtr offset length
+                  let inchunk = mkBS inFPtr offset length
                   finish (DecompressStreamEnd inchunk)
 
       Stream.Error code msg -> case code of
@@ -714,7 +715,7 @@ decompressStream format (DecompressParams bits initChunkSize mdict allMembers)
     outputBufferBytesAvailable <- Stream.outputBufferBytesAvailable
     if outputBufferBytesAvailable > 0
       then do (outFPtr, offset, length) <- Stream.popOutputBuffer
-              return (DecompressOutputAvailable (S.PS outFPtr offset length) (return end))
+              return (DecompressOutputAvailable (mkBS outFPtr offset length) (return end))
       else return end
 
   setDictionary :: Stream.DictionaryHash -> Maybe S.ByteString
@@ -902,7 +903,7 @@ decompressStreamST format params =
 
 
     tryFollowingStream :: S.ByteString -> Stream.State s -> ST s (DecompressStream (ST s))
-    tryFollowingStream chunk zstate = 
+    tryFollowingStream chunk zstate =
       case S.length chunk of
       0 -> return $ DecompressInputRequired $ \chunk' -> case S.length chunk' of
          0 -> finaliseStreamEnd S.empty zstate
@@ -924,7 +925,8 @@ decompressStreamST format params =
         if S.length chunk > 1
           then do
             -- have to handle the remaining data in this chunk
-            (DecompressInputRequired next, zstate') <- runStreamST resume zstate
+            x <- runStreamST resume zstate
+            let (DecompressInputRequired next, zstate') = x
             (strm', zstate'') <- runStreamST (next (S.tail chunk)) zstate'
             go strm' zstate'' False
           else do
